@@ -18,6 +18,7 @@ module Database.Persist.Class.PersistStore
     , insertEntity
     , insertRecord
     , ToBackendKey(..)
+    , BackendCompatible(..)
     ) where
 
 import qualified Data.Text as T
@@ -48,6 +49,50 @@ class (HasPersistBackend backend) => IsPersistBackend backend where
     -- to accidentally run a write query against a read-only database.
     mkPersistBackend :: BaseBackend backend -> backend
 
+-- | This class witnesses that two backend are compatible, and that you can
+-- convert from the @sub@ backend into the @sup@ backend. This is similar
+-- to the 'HasPersistBackend' and 'IsPersistBackend' classes, but where you
+-- don't want to fix the type associated with the 'PersistEntityBackend' of
+-- a record.
+--
+-- Generally speaking, where you might have:
+--
+-- @
+-- foo ::
+--   ( 'PersistEntity' record
+--   , 'PeristEntityBackend' record ~ 'BaseBackend' backend
+--   , 'IsSqlBackend' backend
+--   )
+-- @
+--
+-- this can be replaced with:
+--
+-- @
+-- foo ::
+--   ( 'PersistEntity' record,
+--   , 'PersistEntityBackend' record ~ backend
+--   , 'BackendCompatible' 'SqlBackend' backend
+--   )
+-- @
+--
+-- This works for 'SqlReadBackend' because of the @instance 'BackendCompatible' 'SqlBackend' 'SqlReadBackend'@, without needing to go through the 'BaseBackend' type family.
+--
+-- Likewise, functions that are currently hardcoded to use 'SqlBackend' can be generalized:
+--
+-- @
+-- -- before:
+-- asdf :: 'ReaderT' 'SqlBackend' m ()
+-- asdf = pure ()
+--
+-- -- after:
+-- asdf' :: 'BackendCompatible' SqlBackend backend => ReaderT backend m ()
+-- asdf' = withReaderT 'projectBackend' asdf
+-- @
+--
+-- @since 2.7.1
+class BackendCompatible sup sub where
+    projectBackend :: sub -> sup
+
 -- | A convenient alias for common type signatures
 type PersistRecordBackend record backend = (PersistEntity record, PersistEntityBackend record ~ BaseBackend backend)
 
@@ -176,7 +221,7 @@ class
         get key >>= maybe (liftIO $ throwIO $ KeyNotFound $ show key) return
 
 
--- | Same as 'get', but for a non-null (not Maybe) foreign key
+-- | Same as 'get', but for a non-null (not Maybe) foreign key.
 -- Unsafe unless your database is enforcing that the foreign key is valid.
 getJust :: ( PersistStoreRead backend
            , Show (Key record)
@@ -188,6 +233,7 @@ getJust key = get key >>= maybe
   return
 
 -- | Same as 'getJust', but returns an 'Entity' instead of just the record.
+--
 -- @since 2.6.1
 getJustEntity
   :: (PersistEntityBackend record ~ BaseBackend backend
@@ -238,7 +284,7 @@ insertEntity e = do
 
 -- | Like @get@, but returns the complete @Entity@.
 getEntity ::
-    ( PersistStoreWrite backend
+    ( PersistStoreRead backend
     , PersistRecordBackend e backend
     , MonadIO m
     ) => Key e -> ReaderT backend m (Maybe (Entity e))
@@ -247,6 +293,7 @@ getEntity key = do
     return $ fmap (key `Entity`) maybeModel
 
 -- | Like 'insertEntity' but just returns the record instead of 'Entity'.
+--
 -- @since 2.6.1
 insertRecord
   :: (PersistEntityBackend record ~ BaseBackend backend
@@ -257,4 +304,3 @@ insertRecord
 insertRecord record = do
   insert_ record
   return $ record
-
