@@ -1,8 +1,11 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE FlexibleContexts #-}
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-module Database.Persist.Sql.Orphan.PersistUnique () where
+{-# OPTIONS_GHC -fno-warn-orphans  #-}
+
+module Database.Persist.Sql.Orphan.PersistUnique
+  ()
+  where
 
 import Control.Exception (throwIO)
 import Control.Monad.IO.Class (liftIO, MonadIO)
@@ -17,15 +20,24 @@ import Data.Monoid (mappend, (<>))
 import qualified Data.Conduit.List as CL
 import Control.Monad.Trans.Reader (ask, withReaderT)
 
-defaultUpsert :: (MonadIO m, PersistEntity record, PersistUniqueWrite backend
-                 , PersistEntityBackend record ~ BaseBackend backend) 
-                => record -> [Update record] -> ReaderT backend m (Entity record)
+defaultUpsert
+    :: (MonadIO m
+       ,PersistEntity record
+       ,PersistUniqueWrite backend
+       ,PersistEntityBackend record ~ BaseBackend backend)
+    => record -> [Update record] -> ReaderT backend m (Entity record)
 defaultUpsert record updates = do
-  uniqueKey <- onlyUnique record
-  upsertBy uniqueKey record updates
+    uniqueKey <- onlyUnique record
+    upsertBy uniqueKey record updates
+
+escape :: DBName -> T.Text
+escape (DBName s) = T.pack $ '"' : escapeQuote (T.unpack s) ++ "\""
+  where
+    escapeQuote "" = ""
+    escapeQuote ('"':xs) = "\"\"" ++ escapeQuote xs
+    escapeQuote (x:xs) = x : escapeQuote xs
 
 instance PersistUniqueWrite SqlBackend where
-
     upsert record updates = do
       conn <- ask
       uniqueKey <- onlyUnique record
@@ -38,10 +50,10 @@ instance PersistUniqueWrite SqlBackend where
                                     vals = (map toPersistValue $ toPersistFields record) ++ (map updatePersistValue updates) ++ (unqs uniqueKey)
                                            
                                     go'' n Assign = n <> "=?"
-                                    go'' n Add = T.concat [n, "=", n, "+?"]
-                                    go'' n Subtract = T.concat [n, "=", n, "-?"]
-                                    go'' n Multiply = T.concat [n, "=", n, "*?"]
-                                    go'' n Divide = T.concat [n, "=", n, "/?"]
+                                    go'' n Add = T.concat [n, "=", escape (entityDB t) <> ".", n, "+?"]
+                                    go'' n Subtract = T.concat [n, "=", escape (entityDB t) <> ".", n, "-?"]
+                                    go'' n Multiply = T.concat [n, "=", escape (entityDB t) <> ".", n, "*?"]
+                                    go'' n Divide = T.concat [n, "=", escape (entityDB t) <> ".", n, "/?"]
                                     go'' _ (BackendSpecificUpdate up) = error $ T.unpack $ "BackendSpecificUpdate" `Data.Monoid.mappend` up `mappend` "not supported"
                                               
                                     go' (x, pu) = go'' (connEscapeName conn x) pu
@@ -63,54 +75,62 @@ instance PersistUniqueWrite SqlBackend where
         t = entityDef $ dummyFromUnique uniq
         go = map snd . persistUniqueToFieldNames
         go' conn x = connEscapeName conn x `mappend` "=?"
-        sql conn = T.concat
-            [ "DELETE FROM "
-            , connEscapeName conn $ entityDB t
-            , " WHERE "
-            , T.intercalate " AND " $ map (go' conn) $ go uniq
-            ]
+        sql conn =
+            T.concat
+                [ "DELETE FROM "
+                , connEscapeName conn $ entityDB t
+                , " WHERE "
+                , T.intercalate " AND " $ map (go' conn) $ go uniq]
+
 instance PersistUniqueWrite SqlWriteBackend where
     deleteBy uniq = withReaderT persistBackend $ deleteBy uniq
 
 instance PersistUniqueRead SqlBackend where
     getBy uniq = do
         conn <- ask
-        let sql = T.concat
-                [ "SELECT "
-                , T.intercalate "," $ dbColumns conn t
-                , " FROM "
-                , connEscapeName conn $ entityDB t
-                , " WHERE "
-                , sqlClause conn
-                ]
+        let sql =
+                T.concat
+                    [ "SELECT "
+                    , T.intercalate "," $ dbColumns conn t
+                    , " FROM "
+                    , connEscapeName conn $ entityDB t
+                    , " WHERE "
+                    , sqlClause conn]
             uvals = persistUniqueToValues uniq
-        withRawQuery sql uvals $ do
-            row <- CL.head
-            case row of
-                Nothing -> return Nothing
-                Just [] -> error "getBy: empty row"
-                Just vals -> case parseEntityValues t vals of
-                    Left err -> liftIO $ throwIO $ PersistMarshalError err
-                    Right r -> return $ Just r
+        withRawQuery sql uvals $
+            do row <- CL.head
+               case row of
+                   Nothing -> return Nothing
+                   Just [] -> error "getBy: empty row"
+                   Just vals ->
+                       case parseEntityValues t vals of
+                           Left err ->
+                               liftIO $ throwIO $ PersistMarshalError err
+                           Right r -> return $ Just r
       where
         sqlClause conn =
             T.intercalate " AND " $ map (go conn) $ toFieldNames' uniq
         go conn x = connEscapeName conn x `mappend` "=?"
         t = entityDef $ dummyFromUnique uniq
         toFieldNames' = map snd . persistUniqueToFieldNames
+
 instance PersistUniqueRead SqlReadBackend where
     getBy uniq = withReaderT persistBackend $ getBy uniq
+
 instance PersistUniqueRead SqlWriteBackend where
     getBy uniq = withReaderT persistBackend $ getBy uniq
 
 dummyFromUnique :: Unique v -> Maybe v
 dummyFromUnique _ = Nothing
 
-
-updateFieldDef :: PersistEntity v => Update v -> FieldDef
+updateFieldDef
+    :: PersistEntity v
+    => Update v -> FieldDef
 updateFieldDef (Update f _ _) = persistFieldDef f
-updateFieldDef (BackendUpdate {}) = error "updateFieldDef did not expect BackendUpdate"
+updateFieldDef (BackendUpdate{}) =
+    error "updateFieldDef did not expect BackendUpdate"
 
 updatePersistValue :: Update v -> PersistValue
 updatePersistValue (Update _ v _) = toPersistValue v
-updatePersistValue (BackendUpdate {}) = error "updatePersistValue did not expect BackendUpdate"
+updatePersistValue (BackendUpdate{}) =
+    error "updatePersistValue did not expect BackendUpdate"
